# -*- coding: utf-8 -*-
"""
SynthTransactions
A model to create synthetic credit card transaction datasets

v 1.0
"""


import math
import matplotlib.pyplot as plt



def CheckDistributionH( Transactions ):
    
    HourDistr = [ 0.0 for k in range(24) ]
    for trans in Transactions:
        h = trans[2]
        HourDistr[h] += 1.0
        
    plt.bar( range(24), HourDistr, align='center' )
    
    return HourDistr
    
    

def CheckDistributionD( Transactions ):
    
    DailyDistr = [ 0.0 for k in range(7) ]
    for trans in Transactions:
        d = trans[0] % 7
        DailyDistr[d] += 1.0
        
    plt.bar( range(7), DailyDistr, align='center' )
    
    return DailyDistr
    
    

def CheckDistributionNumOps( Transactions ):
    
    Users = []
    for trans in Transactions:
        if trans[1] not in Users:
            Users.append( trans[1] )
    
    NumOps = [ 0 for k in range( max(Users) + 1 ) ]
    for trans in Transactions:
        if trans[0] > 30:
            continue
        NumOps[ trans[1] ] += 1
        
    NumOpsDistr = [ 0.0 for k in range(100) ]
    for op in NumOps:
        if op < 100:
            NumOpsDistr[ op ] += 1.0
        
    plt.bar( range(100), NumOpsDistr, align='center' )
    
    return NumOpsDistr

    

def CheckDistributionNumOpsStores( Transactions ):
    
    Stores = []
    for trans in Transactions:
        if trans[4] not in Stores:
            Stores.append( trans[1] )
    
    NumOps = [ 0 for k in range( max(Stores) + 1 ) ]
    for trans in Transactions:
        if trans[0] > 30:
            continue
        NumOps[ trans[4] ] += 1
        
    NumOpsDistr = [ 0.0 for k in range(50) ]
    for op in NumOps:
        if op < 1000:
            NumOpsDistr[ int( op / 20 ) ] += 1.0
        
    plt.bar( range(50), NumOpsDistr, align='center' )
    return NumOpsDistr

    

def CheckTimeBetweenTransactions( Transactions, tNumOps ):
    
    Users = []
    for trans in Transactions:
        if trans[1] not in Users:
            Users.append( trans[1] )
    
    NumOps = [ 0 for k in range( max(Users) + 1 ) ]
    for trans in Transactions:
        NumOps[ trans[1] ] += 1
        
    targetUsers = []
    for user in range( len(NumOps) ):
        if NumOps[ user ] == tNumOps:
            targetUsers.append( user )
            
    lastOp = [ 0.0 for k in range( len( targetUsers ) ) ]
        
    TimeDistr = [ 0.0 for k in range(240) ]
    for trans in Transactions:
        if trans[1] not in targetUsers:
            continue

        offset = targetUsers.index( trans[1] )
        timeN = trans[0] * 24 + trans[2]
        timeD = timeN - lastOp[ offset ]
        if lastOp[ offset ] <= 0.0:
            lastOp[ offset ] = timeN
            continue
        lastOp[ offset ] = timeN
        if timeD >= 240:
            continue
        TimeDistr[ abs( int( timeD ) ) ] += 1
        
    plt.bar( range(240), TimeDistr, align='center' )
    
    return TimeDistr

    

def CheckDistributionQ( Transactions ):
    
    QuantDistr = [ 0.0 for k in range(50) ]
    for trans in Transactions:
        h = int( math.floor( trans[3] / 25 ) )
        QuantDistr[h] += 1.0
        
    plt.bar( range(0, 50*25, 25), QuantDistr, align='center' )
    
    return QuantDistr
    